clear all

% add a viscous curl field to simulate a change in the relationship
% betweeen motor commands and observed outcome (to disturb the good brain-stem controller)
phi = 90; % curl in the body/plant dynamics
P1 = [cosd(phi) sind(phi); -sind(phi) cosd(phi)]; % modified plant dynamics
B = [1 0; 0 1]; % brain-stem transformation


% simulation
dt = 1;
t = 0:dt:150; % 200 time-steps

% define stimulus sequence
x_dot = [sin(0.1*t); sin(0.2*t)]; % head velocity

% RUN
u = 0*x_dot;
params.w = zeros(size(x_dot, 1) ,size(x_dot, 1)); % cerebellum output weight parameters
params.lr = 0.1; % learning rate

figure
for i = 1:length(t)-1
    %% generate motor command ut
    cerebellum = cerebellum_compute(x_dot(:, i), params);
    y_c(:,i) = cerebellum.y_c;
    p(:,i) = cerebellum.p;
    
    %% calculate the eye velocity outcome 'y_dot' and observed error 'e'
    %% compute brain-stem output
    u_b = B * x_dot;
    %% total motor command = brainstem + cerebellm outputs
    u(:,i) = u_b(:,i) + cerebellum.y_c;
    %% compute eye velocity
    y_dot(:, i) = P1 * (u(:, i));
    %% retinal slip
    e(:, i) = (x_dot(:, i) - y_dot(:, i));
    
    %% estimate the cerebellar error 'c'
    %c = e(:, i); % works only for small curl-filed values 'phi'
    c = inv(P1) * e(:, i); % works for any 'phi'
    
    %% update cerebellum weights
    params = cerebellum_update(c, cerebellum, params);
    
%     %% plots
%     subplot(1,2,1)
%     plot(i, x_dot(1,i), 'b.', 'MarkerSize', 10)
%     hold on
%     plot(i, y_dot(1,i), 'r.', 'MarkerSize', 10)
%     legend('head vel x', 'eye vel x')
% 
%     subplot(1,2,2)
%     plot(i, x_dot(2,i), 'b.', 'MarkerSize', 10)
%     hold on
%     plot(i, y_dot(2,i), 'r.', 'MarkerSize', 10)
%     legend('head vel y', 'eye vel y')
%     
%     drawnow
%     pause(0.0000001)
end

subplot(1,2,1)
plot(x_dot(1,:), 'b', 'LineWidth', 4)
hold on
plot(y_dot(1,:), 'r', 'LineWidth', 4)
legend('head vel x', 'eye vel x')

subplot(1,2,2)
plot(x_dot(2,:), 'b', 'LineWidth', 4)
hold on
plot(y_dot(2,:), 'r', 'LineWidth', 4)
legend('head vel y', 'eye vel y')

figure
plot(mean(sqrt(e.^2)), 'LineWidth', 4)
xlabel('time steps')
ylabel('retinal slip (e)')





% figure
% plot(x_dot(1,:), x_dot(2, :), 'b', 'LineWidth', 4)
% hold on
% plot(y_dot(1,:), y_dot(2, :), '--r', 'LineWidth', 4)
% legend('head vel', 'eye vel')